/* -*- c++ -*-
 *
 * Copyright (c) 2007 Anthony Mallet                  --  Thu Feb  1 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_V3D
#define H_V3D

#include "t3d/t3d.h"

/* --- API ----------------------------------------------------------- */

/** \addtogroup C */
/** \{ */

/** \brief Vector index values
 *
 * Use this enumerator with V3D coordinates accessors.
 */
typedef enum V3DVectorIndex {
  V3D_VECTOR_X	=	0,	/**<\brief X coordinate. */
  V3D_VECTOR_Y =	1,	/**<\brief Y coordinate. */
  V3D_VECTOR_Z =	2	/**<\brief Z coordinate. */
} V3DVectorIndex;

#ifdef __cplusplus
extern "C" {
#endif

  V3D *		v3dNew(void);
  V3D *		v3dInit(V3D *v3d);
  V3D *		v3dDup(V3D *v3d);
  void		v3dDelete(V3D *v3d);

  double	v3dNorm(const V3D *v3d);
  void		v3dNormalize(V3D *v3d);
  V3D *		v3dApplyT3D(T3D *t3d, V3D *v3d);

  double 		v3dDot(const V3D *a, const V3D *b);
  double *		v3dDotIn(double *c, const V3D *a, const V3D *b);

  V3D *		v3dProd(V3D *a, const V3D *b);
  V3D *		v3dProdIn(V3D *c, const V3D *a, const V3D *b);

  int		v3dFPrint(FILE *stream, const V3D *v3d);

#ifdef __cplusplus
}
#endif

/** \} */

/* --- V3D data structures ------------------------------------------- */

/** \brief A 3D vector
 *
 * This structure manipulates vectors represented by homogeneous
 * coordinates.
 */
struct V3D {
#ifdef __cplusplus
 public:
  inline V3D(void);
  inline V3D(double x, double y, double z);

  inline double &operator[](unsigned int i);
  inline double &operator()(unsigned int i);

  inline double operator[](unsigned int i) const;
  inline double operator()(unsigned int i) const;

  inline double &operator[](V3DVectorIndex i);

  inline V3D operator+(const V3D &inV) const;
  inline V3D operator+=(const V3D &inV);
  inline V3D operator-() const;
  inline V3D operator-(const V3D &inV) const;
  inline V3D operator-=(const V3D &inV);
  inline V3D operator/(const double d) const;
  inline V3D operator/=(const double d);
  inline V3D operator*(const double d) const;
  inline V3D operator*=(const double d);
  inline double operator*(const V3D& inV) const;
  inline V3D operator^(const V3D& inV) const;
  inline bool operator==(const V3D& inV) const;

  inline double norm() const;
  inline V3D &normalize();

  friend std::ostream& operator<<(std::ostream &os, const V3D &v3d);
#endif

  double v[4];	/**<\brief Homogeneous coordinates. */
};

#ifdef __cplusplus
/* include C++ inlined methods */
# include "t3d/v3d_inline.h"
#endif

#endif /* H_V3D */
